/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.viewmodel;

import jalview.api.AlignViewportI;
import jalview.datamodel.AlignmentI;

/**
 * Supplies and updates viewport properties relating to position such as: start
 * and end residues and sequences, hidden column/row adjustments, ratio of
 * viewport to alignment etc
 */
public class ViewportPositionProps extends ViewportProperties
{
  // start residue of viewport
  private int startRes;

  // end residue of viewport
  private int endRes;

  // start sequence of viewport
  private int startSeq;

  // end sequence of viewport
  private int endSeq;

  // alignment
  private AlignmentI al;

  // viewport
  private AlignViewportI av; // this is a bad dependency, viewmodel should not
                             // depend on api

  /**
   * Constructor
   * @param alignment TODO
   */
  public ViewportPositionProps(AlignmentI alignment, AlignViewportI viewport)
  {
    // initial values of viewport settings
    this.startRes = 0;
    this.endRes = alignment.getWidth() - 1;
    this.startSeq = 0;
    this.endSeq = alignment.getHeight() - 1;
    this.al = alignment;
    this.av = viewport;
  }

  // ways to update values

  // ways to notify of changes

  // ways to supply positional information

  /**
   * Get alignment width
   */
  public int getAlignmentWidthInCols()
  {
    return al.getWidth();
  }

  /**
   * Get alignment height
   */
  public int getAlignmentHeightInRows()
  {
    return al.getHeight();
  }

  public void setStartRes(int res)
  {
    if (res > al.getWidth() - 1)
    {
      res = al.getWidth() - 1;
    }
    else if (res < 0)
    {
      res = 0;
    }
    this.startRes = res;
  }

  public void setEndRes(int res)
  {
    if (res > al.getWidth())
    {
      res = al.getWidth();
    }
    else if (res < 1)
    {
      res = 1;
    }
    this.endRes = res;
  }

  public void setStartSeq(int seq)
  {
    if (seq > al.getHeight() - 1)
    {
      seq = al.getHeight() - 1;
    }
    else if (seq < 0)
    {
      seq = 0;
    }
    this.startSeq = seq;
  }

  public void setEndSeq(int seq)
  {
    if (seq > al.getHeight())
    {
      seq = al.getHeight();
    }
    else if (seq < 1)
    {
      seq = 1;
    }
    this.endSeq = seq;
  }

  /**
   * Get start residue of viewport
   */
  public int getStartRes()
  {
    return startRes;
  }

  /**
   * Get end residue of viewport
   */
  public int getEndRes()
  {
    return endRes;
  }

  /**
   * Get start sequence of viewport
   */
  public int getStartSeq()
  {
    return startSeq;
  }

  /**
   * Get end sequence of viewport
   */
  public int getEndSeq()
  {
    return endSeq;
  }

  /**
   * Get absolute start residue of viewport
   */
  public int getAbsoluteStartRes()
  {
    int start = startRes;

    if (av.hasHiddenColumns())
    {
      start = av.getColumnSelection().adjustForHiddenColumns(start);
    }
    return start;
  }

  /**
   * Get absolute start residue of viewport
   */
  public int getAbsoluteEndRes()
  {
    int end = endRes;

    if (av.hasHiddenColumns())
    {
      end = av.getColumnSelection().adjustForHiddenColumns(end);
    }
    return end;
  }

  /**
   * Get absolute start sequence of viewport
   */
  public int getAbsoluteStartSeq()
  {
    int start = startSeq;

    if (av.hasHiddenRows())
    {
      start = av.getAlignment().getHiddenSequences()
              .adjustForHiddenSeqs(start);
    }
    return start;
  }

  /**
   * Get absolute end sequence of viewport
   */
  public int getAbsoluteEndSeq()
  {
    int end = endSeq;

    if (av.hasHiddenRows())
    {
      end = av.getAlignment().getHiddenSequences().adjustForHiddenSeqs(end);
    }
    return end;
  }

}
